"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.linkProjectToSession = linkProjectToSession;

var _core = require("@yarnpkg/core");

var _get = _interopRequireDefault(require("lodash/get"));

var _tauProlog = _interopRequireDefault(require("tau-prolog"));

var _vm = _interopRequireDefault(require("vm"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/// <reference path="./tauProlog.d.ts"/>
// eslint-disable-next-line @typescript-eslint/camelcase
const {
  is_atom: isAtom,
  is_variable: isVariable,
  is_instantiated_list: isInstantiatedList
} = _tauProlog.default.type;

function prependGoals(thread, point, goals) {
  thread.prepend(goals.map(goal => new _tauProlog.default.type.State(point.goal.replace(goal), point.substitution, point)));
}

const projects = new WeakMap();

function getProject(thread) {
  const project = projects.get(thread.session);
  if (project == null) throw new Error(`Assertion failed: A project should have been registered for the active session`);
  return project;
}

const tauModule = new _tauProlog.default.type.Module(`constraints`, {
  [`project_workspaces_by_descriptor/3`]: (thread, point, atom) => {
    const [descriptorIdent, descriptorRange, workspaceCwd] = atom.args;

    if (!isAtom(descriptorIdent) || !isAtom(descriptorRange)) {
      thread.throwError(_tauProlog.default.error.instantiation(atom.indicator));
      return;
    }

    const ident = _core.structUtils.parseIdent(descriptorIdent.id);

    const descriptor = _core.structUtils.makeDescriptor(ident, descriptorRange.id);

    const project = getProject(thread);
    const workspace = project.tryWorkspaceByDescriptor(descriptor);

    if (isVariable(workspaceCwd)) {
      if (workspace !== null) {
        prependGoals(thread, point, [new _tauProlog.default.type.Term(`=`, [workspaceCwd, new _tauProlog.default.type.Term(String(workspace.relativeCwd))])]);
      }
    }

    if (isAtom(workspaceCwd)) {
      if (workspace !== null && workspace.relativeCwd === workspaceCwd.id) {
        thread.success(point);
      }
    }
  },
  [`workspace_field/3`]: (thread, point, atom) => {
    const [workspaceCwd, fieldName, fieldValue] = atom.args;

    if (!isAtom(workspaceCwd) || !isAtom(fieldName)) {
      thread.throwError(_tauProlog.default.error.instantiation(atom.indicator));
      return;
    }

    const project = getProject(thread);
    const workspace = project.tryWorkspaceByCwd(workspaceCwd.id); // Workspace not found => this predicate can never match
    // We might want to throw here? We can be pretty sure the user did
    // something wrong at this point

    if (workspace == null) return;
    const value = (0, _get.default)(workspace.manifest.raw, fieldName.id); // Field is not present => this predicate can never match

    if (typeof value === `undefined`) return;
    prependGoals(thread, point, [new _tauProlog.default.type.Term(`=`, [fieldValue, new _tauProlog.default.type.Term(String(value))])]);
  },
  [`workspace_field_test/3`]: (thread, point, atom) => {
    const [workspaceCwd, fieldName, checkCode] = atom.args;
    thread.prepend([new _tauProlog.default.type.State(point.goal.replace(new _tauProlog.default.type.Term(`workspace_field_test`, [workspaceCwd, fieldName, checkCode, new _tauProlog.default.type.Term(`[]`, [])])), point.substitution, point)]);
  },
  [`workspace_field_test/4`]: (thread, point, atom) => {
    const [workspaceCwd, fieldName, checkCode, checkArgv] = atom.args;

    if (!isAtom(workspaceCwd) || !isAtom(fieldName) || !isAtom(checkCode) || !isInstantiatedList(checkArgv)) {
      thread.throwError(_tauProlog.default.error.instantiation(atom.indicator));
      return;
    }

    const project = getProject(thread);
    const workspace = project.tryWorkspaceByCwd(workspaceCwd.id); // Workspace not found => this predicate can never match
    // We might want to throw here? We can be pretty sure the user did
    // something wrong at this point

    if (workspace == null) return;
    const value = (0, _get.default)(workspace.manifest.raw, fieldName.id); // Field is not present => this predicate can never match

    if (typeof value === `undefined`) return; // Inject the variables into a sandbox

    const vars = {
      $$: value
    };

    for (const [index, value] of checkArgv.toJavaScript().entries()) vars[`$${index}`] = value;

    const result = _vm.default.runInNewContext(checkCode.id, vars);

    if (result) {
      thread.success(point);
    }
  }
}, [`project_workspaces_by_descriptor/3`, `workspace_field/3`, `workspace_field_test/3`, `workspace_field_test/4`]);

function linkProjectToSession(session, project) {
  projects.set(session, project);
  session.consult(`:- use_module(library(${tauModule.id})).`);
}